const fs = require('fs');
const path = require('path');
const ms = require('ms');
const moment = require('moment');

const DateHandler = require('../functions/DateHandler');
const Logger = require('./Logger');
const SystemLogger = require('./SystemLogger');

class UpdaterLogger extends Logger {

  constructor() {
    super({
      filePath: process.env.UPDATER_LOG_FOLDER,
      fileExtension: 'zip'
    });
    this.hideConsoleLogger = true;
  }

  format(zipContent) {
    return zipContent;
  }

  async writeZip(CDFILIAL, CDCAIXA, zipContent) {
    this.fileName = `assistant_logs_filial_${CDFILIAL}`;
    if(CDCAIXA) this.fileName = `${this.fileName}_cx_${CDCAIXA}`;
    if (!fs.existsSync(this.filePath)) {
      fs.mkdirSync(this.filePath, { recursive: true });
    }
    let response = {};
    try {
      const { fileName } = await this.write(zipContent, {}, { useWriteFile: true });
      response = {
        error: false,
        message: null,
        fileName
      }
    } catch (error) {
      response = { 
        error: true, 
        message: error.message, 
        fileName: null
      }
    }
    return response;
  }

  async getLocalLogInfo(CDFILIAL, CDCAIXA) {
    return new Promise((resolve, reject) => {
      const sysdate = DateHandler.toString();
      const timeKeepLog = ms(process.env.TIME_KEEP_UPDATER_LOG);
      let fileName = `assistant_logs_filial_${CDFILIAL}`;
      if(CDCAIXA) fileName = `${fileName}_cx_${CDCAIXA}`;
      fileName = `${fileName}_${sysdate}.zip`;
      fs.stat(path.join(this.filePath, fileName), (err, stats) => {
        let modifiedTime = err ? null : stats.mtime.getTime();
        let lapsedTime = Date.now() - (modifiedTime || 0);
        const useLocalLog = !err && lapsedTime < timeKeepLog;
        resolve({
          useLocalLog,
          fileName
        });
      });
    });
  }

  async removeFile(filePath, fileName) {
    return new Promise((resolve, reject) => {
      fs.unlink(path.join(filePath, fileName), (err) => {
        if (err) return reject(err);
        resolve();
      });
    })
  }

  isOldLog(logDate) {
    return moment(logDate, 'YYYY-MM-DD') < moment(moment().format('YYYY-MM-DD'));
  }

  async clearLogs() {
    const DATE_PATTERN = /\d{4}-\d{2}-\d{2}/i;
    return new Promise((resolve, reject) => {
      const logFolder = process.env.UPDATER_LOG_FOLDER;
      if(!logFolder) {
        return SystemLogger.error('<UPDATER_LOG_FOLDER> not informed');
      }
      if (!fs.existsSync(logFolder)) {
        fs.mkdirSync(logFolder, { recursive: true });
      }
      fs.readdir(logFolder, async (err, files) => {
        if (err) return reject(err);
        const allPromiseRemoveFiles = [];
        files.forEach(fileName => {
          const [ logDate ] = fileName.match(DATE_PATTERN) || [];
          if(logDate && this.isOldLog(logDate)) {
            allPromiseRemoveFiles.push(
              this.removeFile(logFolder, fileName)
            );
          }
        });
        const response = await Promise.allSettled(allPromiseRemoveFiles);
        const errors = response.filter(result => result.status  === 'rejected');
        if(errors.length) {
          const errorMessages = errors.map(err => err.reason);
          const message = `Ocorreram erros ao remover arquivos de log do updater: ${errorMessages.join('\n')}`;
          SystemLogger.error(message);
          return resolve({error: true, message})
        }
        resolve({error: false});
      });
    });
  }

  startCleanupTask() {
    this.clearLogs();
    const intervalId = setInterval(() => {
      this.clearLogs();
    }, ms('1d'));
  }

}

module.exports = new UpdaterLogger();